//******************************************************************************
//
//                 UNIT FINDCOMPMENUWIZARD (TCompRefMenuWizard)
//
//******************************************************************************
// Project:        DELPHI IDE PLUGINS
//                 FIND COMPONENT REFERENCE WIZARD
// File:           FindCompMenuWizard.pas
// Directory:      \FindCompRef\Source\Wizard\
// Function:       Wizard to install PasLib menu item on Delphi Project menu
// Description:    This wizard is run once during initial loading of the wizard,
//                 in order to install a menu item on the Delphi IDE Search menu
//                 that is linked to the TFindCompRefWizard.Execute method.
// Author:         Kristofer G. Skaug <kristofer@xs4all.nl>
// Environment:    Delphi 6.2, P4/1700MHz, 512MB RAM, Windows XP SP1
// Compiled state: Delphi compiled unit: FindCompMenuWizard.dcu
// Resources:      Win32 API
// Notes:          ..
//
// Revisions:
//
// 01.00.00 20/04/2003 First Version
//
//******************************************************************************

unit FindCompMenuWizard;

interface

uses SysUtils,Classes,Menus,Forms,ExtCtrls,ToolsAPI,
// Shared:
FindCompRegWizard;

//------------------------------------------------------------------------------
// Single public factory function as interface:
//------------------------------------------------------------------------------
function MenuWizard(const AWizard:IOTAWizard):IOTAWizard;


implementation

//------------------------------------------------------------------------------
// Local declaration, hides class from outside access:
type TCompRefMenuWizard = class(TBaseWizard)
private
  FMenuItem: TMenuItem;
  FItemName: string;
  FShortCut: TShortCut;
  FWizard: IOTAWizard;
  FShortCutTimer: TTimer;
protected
  procedure InstallShortCut(Sender: TObject);
  procedure FindCompRefItemClick(Sender: TObject);
public
  constructor Create(const AWizard:IOTAWizard);
  destructor Destroy; override;
  function GetName:string; override; // promoted visibility from protected
end;


//******************************************************************************
// Method Name:      MenuWizard
// Input parameters: AWizard: reference to TFindCompRefWizard interface
// Return value:     IOTAWizard
// Exceptions:       ..  
// Side effects:     ..
// Conditions:       ..
// Description:      Menu Wizard entry point; TCompRefMenuWizard factory function.
// Notes:            Single publicly accessible point of this menu wizard.
//******************************************************************************
function MenuWizard(const AWizard:IOTAWizard):IOTAWizard;
begin
  Result:=TCompRefMenuWizard.Create(AWizard);
end; // func MenuWizard

//******************************************************************************
// Method Name:      TCompRefMenuWizard.Create
// Input parameters: AWizard: reference to TFindCompRefWizard interface
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Creates a submenu item on the Delphi "Search" menu and
//                   assigns the OnClick handler to AWizard.Execute.
// Notes:            ..
//******************************************************************************
constructor TCompRefMenuWizard.Create(const AWizard:IOTAWizard);
const SearchMenuName = 'SearchMenu0';
      SubItemCaption = '&Find Selected Component in Source/Form...';
      SubItemName = 'FindCompRefItem';
var NTAServices: INTAServices;
    SearchMenu: TMenuItem;

  // Return handle to Delphi IDE menu's "Search" menu item
  function FindSearchMenu: TMenuItem;
  var idx: integer;
  begin
    Result:=nil;
    for idx:=0 to NTAServices.MainMenu.Items.Count-1 do
    if (NTAServices.MainMenu.Items[idx].Name=SearchMenuName) then
    begin
      Result:=NTAServices.MainMenu.Items[idx];
      break;
    end;
    if Result=nil then Result:=NTAServices.MainMenu.Items[2]; // Brute-force way.
  end; // func FindSearchMenu

begin
  inherited Create;
  FWizard:=AWizard; // save reference in local field
  FItemName:=SubItemName;
  FShortCut:=TextToShortCut('Ctrl+Shift+F');
  if Supports(BorlandIDEServices,INTAServices,NTAServices)
    and Assigned(NTAServices.MainMenu) then
  begin
    SearchMenu:=FindSearchMenu;
    if Assigned(SearchMenu) then
    begin
      // Create New Submenu:
      FMenuItem:=Menus.NewItem(SubItemCaption, FShortCut, False, True,
                                     FindCompRefItemClick, 0, FItemName);
      // Insert submenu at bottom of Search menu:
      SearchMenu.Insert(SearchMenu.Count,FMenuItem);

      // Menu Item Shortcut Persistence Problem:
      // According to well-informed sources, the problem of persisting a custom
      // shortcut in the Delphi IDE main menu can be mitigated by using a timer to
      // refresh the shortcut registration after the IDE has completed loading.
      // It doesn't seem to work 100% waterproof with D6, but it's an improvement.
      FShortCutTimer:=TTimer.Create(Application);
      FShortCutTimer.Interval:=5000; // 5 sec should be enough to ensure all's OK?
      FShortCutTimer.OnTimer:=InstallShortCut;

    end
    else raise Exception.Create('Failed to Insert Item in Search Menu!');
  end; // if Supports() and Assigned()
end; // proc Create

//******************************************************************************
// Method Name:      TCompRefMenuWizard.InstallShortCut
// Input parameters: Sender:TObject
// Exceptions:       ..
// Side effects:     ..
// Conditions:       FShortCutTimer.OnTimer event (executed exactly once!)
// Description:      Installs Shortcut on menu - then destroys timer
// Notes:            ..
//******************************************************************************
procedure TCompRefMenuWizard.InstallShortCut(Sender:TObject);
begin
  FMenuItem.ShortCut:=FShortCut;
  FShortCutTImer.OnTimer:=nil;
  FShortCutTimer.Free;
  FShortCutTimer:=nil;
end; // proc InstallShortCut

//******************************************************************************
// Method Name:      TCompRefMenuWizard.Destroy
// Input parameters: None
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Destructor for Menu wizard; removes the custom Menu item
// Notes:            ..
//******************************************************************************
destructor TCompRefMenuWizard.Destroy;
begin
  if Assigned(FMenuItem) then
  begin
    // Free submenu items, if applicable (someone else might've added them):
    while FMenuItem.Count>0 do FMenuItem.Items[0].Free;
    // Menu items remove themselves from parent menu as part of destructor
    // action, so FMenuItem.Count will be consequently adjusted for each iteration.
    FMenuItem.Free;
  end; // if Assigned()
  inherited Destroy;
end; // proc Destroy

//******************************************************************************
// Method Name:      TCompRefMenuWizard.FindCompRefItemClick
// Input parameters: Sender: TObject
// Exceptions:       ..
// Side effects:     ..
// Conditions:       OnClick event of FMenuItem
// Description:      Invokes the Wizard function (TFindCompRefWizard.Execute)
// Notes:            ..
//******************************************************************************
procedure TCompRefMenuWizard.FindCompRefItemClick(Sender: TObject);
begin // Invoke wizard:
  if Assigned(FWizard) then FWizard.Execute;
end; // proc FindCompRefItemClick

//******************************************************************************
// Method Name:      TCompRefMenuWizard.GetName
// Input parameters: None
// Return value:     string Name
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Mandatory implementation for GetName
// Notes:            ..
//******************************************************************************
function TCompRefMenuWizard.GetName:string;
begin
  Result:='FindCompRef Menu Wizard';
end; // func GetName


end.

//******************************************************************************
//*                          End of File FindCompMenuWizard.pas
//******************************************************************************



