//******************************************************************************
//
//                 UNIT FINDCOMPREGWIZARD (TFCRRegisterWizard)
//
//******************************************************************************
// Project:        DELPHI IDE PLUGINS
//                 FIND COMPONENT REFERENCE WIZARD
// File:           FindCompRegWizard.pas
// Directory:      \FindCompRef\Source\Wizard\
// Function:       Base and Registration classes
// Description:    The TBaseWizard is the base class for the other wizard classes
//                 in this wizard package, implementing common stub functions.
//                 The TFCRRegisterWizard ensures that the FindCompRef Wizard
//                 including the menu item wizard is installed in the Delphi IDE
// Author:         Kristofer G. Skaug <kristofer@xs4all.nl>
// Environment:    Delphi 6.2, P4/1700MHz, 512MB RAM, Windows XP SP1
// Compiled state: Delphi compiled unit: FindCompRegWizard.dcu
// Resources:      Win32 API
// Notes:          ..
//
// Revisions:
//
// 01.00.00 20/04/2003 First Version
//
//******************************************************************************

unit FindCompRegWizard;

interface

uses ToolsApi;

type TBaseWizard = class(TNotifierObject, IOTAWizard)
public
  function GetIDString: string; virtual;
  function GetName: string; virtual; abstract;
  function GetState: TWizardState; virtual;
  procedure Execute; virtual;
end;

procedure Register;

implementation

uses SysUtils,
     FindCompWizard,FindCompMenuWizard;

{ TBaseWizard }

//******************************************************************************
// Method Name:      TBaseWizard.GetIDString
// Input parameters: None
// Return value:     string
// Exceptions:       ..  
// Side effects:     ..
// Conditions:       ..
// Description:      Unique ID string for wizard class
// Notes:            Mandatory implementation for IOTANotifier interface
//******************************************************************************
function TBaseWizard.GetIDString: string;
begin
  Result:='KGSkaug.FCRWiz.'+ClassName;
end; // func GetIDString

//******************************************************************************
// Method Name:      TBaseWizard.GetState
// Input parameters: None
// Return value:     TWizardState
// Exceptions:       ..  
// Side effects:     ..
// Conditions:       ..
// Description:      Stub function - apparently this is never called by the IDE
// Notes:            Mandatory implementation for IOTANotifier interface
//******************************************************************************
function TBaseWizard.GetState: TWizardState;
begin
  Result:=[wsEnabled];
end; // func GetState

//******************************************************************************
// Method Name:      TBaseWizard.Execute
// Input parameters: None
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Performs the main function of the wizard.
// Notes:            Empty virtual base class declaration.
//******************************************************************************
procedure TBaseWizard.Execute;
begin
  // Do nothing here
end; // proc Execute

//------------------------------------------------------------------------------
//  Local declaration, hides external class access:
//------------------------------------------------------------------------------
type TFCRRegisterWizard = class(TBaseWizard)
private
  // Handles to the registered wizards:
  FFindWizardHandle,FMenuWizardHandle: integer;
protected
  function GetName:string; override;
public
  constructor Create;
  destructor Destroy; override;
end;

//******************************************************************************
// Method Name:      Register
// Input parameters: None
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Registers the Registration Wizard (TFCRRegisterWizard)
// Notes:            ..
//******************************************************************************
procedure Register;
begin
  RegisterPackageWizard(TFCRRegisterWizard.Create);
end; // proc Register

//******************************************************************************
// Method Name:      TFCRRegisterWizard.Create
// Input parameters: None
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Constructor, creates and installs Menu and Import wizards
// Notes:            ..
//******************************************************************************
constructor TFCRRegisterWizard.Create;
var AWizard:IOTAWizard;
    WizardServices:IOTAWizardServices;
begin
  inherited;
  if Supports(BorlandIDEServices,IOTAWizardServices,WizardServices) then
  begin
    AWizard:=FindCompWizard.FindCompRefWizard; // factory function
    if Assigned(AWizard) then
    begin
      FFindWizardHandle:=WizardServices.AddWizard(AWizard);
      FMenuWizardHandle:=WizardServices.AddWizard(MenuWizard(AWizard));
    end;
  end;
end; // proc Create

//******************************************************************************
// Method Name:      TFCRRegisterWizard.Destroy
// Input parameters: None
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Destructor, cleans up Menu and Import wizards
// Notes:            ..
//******************************************************************************
destructor TFCRRegisterWizard.Destroy;
var WizardServices:IOTAWizardServices;
begin
  if Supports(BorlandIDEServices,IOTAWizardServices,WizardServices) then
  begin
    WizardServices.RemoveWizard(FMenuWizardHandle);
    WizardServices.RemoveWizard(FFindWizardHandle);
  end;
  inherited;
end; // proc Destroy

//******************************************************************************
// Method Name:      TFCRRegisterWizard.GetName
// Input parameters: None
// Return value:     string Display name
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Mandatory implementation for abstract GetName override
// Notes:            ..
//******************************************************************************
function TFCRRegisterWizard.GetName: string;
begin
  Result:='FindCompRef Registration Wizard';
end; // proc GetName


end.

//******************************************************************************
//*                            End of File FindCompRegWizard.pas
//******************************************************************************



