//******************************************************************************
//
//                 UNIT FINDCOMPWIZARD (TFindCompRefWizard)
//
//******************************************************************************
// Project:        DELPHI IDE PLUGINS
//                 FIND COMPONENT REFERENCE WIZARD
// File:           FindCompWizard.pas
// Directory:      \FindCompRef\Source\Wizard\
// Function:       Core nonvisual invocation class for PasLib Import Wizard
// Description:    This unit implements the core functionality of the Wizard;
//                 The TFindCompRefWizard.Execute method Seeks the topmost editor
//                 in the IDE; and decides on the circumstance:
//                 - If it's a Form editor, invokes "Find comp ref in source".
//                 - If it's a Source editor, invokes "Find component in form"
// Author:         Kristofer G. Skaug <kristofer@xs4all.nl>
// Environment:    Delphi 6.2, P4/1700MHz, 512MB RAM, Windows XP SP1
// Compiled state: Delphi compiled unit: FindCompWizard.dcu
// Resources:      Win32 API
// Notes:          Known problems in v.01.00.00 (for D6):
//                 - The Search function does not actually recall the automated
//                   calls made by this wizard; therefore, hitting <F3> for
//                   "Search Again" after the wizard function has been invoked
//                   for "Find comp ref in source" function, does not work
//                   (i.e. the component name does not stay in IDE search buffer)
//                 - Selecting a Form works if form name is used in source, but
//                   not vice versa (Form name in source does not bring up form editor)
//                 - The wizard cannot see if an identifier reference is inside a
//                   comment, so cannot guarantee that the skip goes to live source.
//                 - Going from form to source, the wizard cannot distinguish
//                   between components of the same name in different frames
//                   (eg. Form1.Label1/Form1.Frame21.Label1/Form1.Frame22.Label1).
//
// Revisions:
//
// 01.00.00 20/04/2003 First Version
//
//******************************************************************************

unit FindCompWizard;

interface

uses ToolsAPI,FindCompRegWizard;

//------------------------------------------------------------------------------
// Single public factory function as interface to this wizard:
//------------------------------------------------------------------------------
function FindCompRefWizard:IOTAWizard;

implementation

uses SysUtils,Dialogs,Classes;

//------------------------------------------------------------------------------
// LOCAL UTILITY FUNCTIONS
//------------------------------------------------------------------------------

//******************************************************************************
// Method Name:      CurrentIdent
// Input parameters: Source: string containing entire source buffer
//                   CurPos: Current cursor position in source var : Integer
// Output params:    Pos: Start position of current identifier
//                   Len: Length of current identifier
// Return value:     Boolean TRUE if we have an identifier, FALSE otherwise
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      This function determines the location and length of the
//                   "current identifier" given current cursor position and source.
//                   The actual identifier value can then be easily extracted
//                   using Copy(Source,Pos,Len);
// Notes:            Code Borrowed from GExperts: GX_ePrevNextIdentifier.pas
//******************************************************************************
function CurrentIdent(const Source:string; CurPos:integer; var Pos,Len:integer):boolean;
const IdentifierChars      = ['A'..'Z', 'a'..'z', '0'..'9', '_'];
begin
  Result := False;
  while CurPos >= 1 do
    if Source[CurPos] in IdentifierChars then
    begin
      Dec(CurPos);
      Result:=True;
    end
    else if (not Result) and (CurPos >= 2) then
      if Source[CurPos - 1] in IdentifierChars then
      begin
        Dec(CurPos,2);
        Result:=True;
      end
      else Break
    else Break;

  if Result then
  begin
    Pos:=CurPos + 1;
    Inc(CurPos, 2);
    while (CurPos >= 1) and (CurPos <= Length(Source)) do
      if Source[CurPos] in IdentifierChars then Inc(CurPos)
      else Break;
    Len := CurPos - Pos;
  end;
end; // func CurrentIdent

//******************************************************************************
// Method Name:      UnitHasForm
// Input parameters: Module: Current IDE module (project)
//                   UnitFileName: Full filename of unit in source editor
// Output params:    FrmEditor: Interface to associated Form editor, if existent
// Return value:     boolean TRUE if associated form found, FALSE otherwise
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Returns the associated form editor interface of a .pas unit
// Notes:            ..
//******************************************************************************
function UnitHasForm(Module:IOTAModule; const UnitFileName:string;
                     out FrmEditor:IOTAFormEditor):boolean;
var i:integer;
    FormFileName:string;
begin
  Result:=False;
  FormFileName:=ChangeFileExt(UnitFileName,'.dfm');
  for i:=0 to Module.GetModuleFileCount-1 do
    if Supports(Module.GetModuleFileEditor(i),IOTAFormEditor,FrmEditor) then
    if SameText(FrmEditor.FileName,FormFileName) then // case-insensitive
    begin
      Result:=True;
      Exit;
    end;
end; // func UnitHasForm

//******************************************************************************
// Method Name:      FormHasUnit
// Input parameters: Module: Current IDE module (project)
//                   FormFileName: Full filename of .dfm unit in form editor
// Output params:    SrcEditor: Interface to associated source editor, if existent
// Return value:     boolean TRUE if associated unit found, FALSE otherwise
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Returns the associated source editor interface of a form
// Notes:            ..
//******************************************************************************
function FormHasUnit(Module:IOTAModule; const FormFileName:string;
                     out SrcEditor:IOTASourceEditor):boolean;
var i:integer;
    UnitFileName:string;
begin
  Result:=False;
  UnitFileName:=ChangeFileExt(FormFileName,'.pas');
  for i:=0 to Module.GetModuleFileCount-1 do
    if Supports(Module.GetModuleFileEditor(i),IOTASourceEditor, SrcEditor) then
    if SameText(SrcEditor.FileName,UnitFileName) then // case-insensitive
    begin
      Result:=True;
      Exit;
    end;
end; // func FormHasUnit

//******************************************************************************
// Method Name:      EditorBufAsString
// Input parameters: Editor: Interface to source editor
// Return value:     string containing all text in editor's code buffer
// Exceptions:       ..  
// Side effects:     ..
// Conditions:       ..
// Description:      Returns current source editor buffer as single string
// Notes:            Code snippet from Gerrit Beuze, ModelMaker inc.
//******************************************************************************
function EditorBufAsString(Editor: IOTASourceEditor): string;
var
  Reader: IOTAEditReader;
  StrStrm: TStringStream;
  Position: Integer;
  NrRead: Integer;
  Buf: array[0..8192] of Char;
begin
  Reader := Editor.CreateReader;
  StrStrm := TStringStream.Create('');
  try
    Position := 0;
    repeat
      NrRead := Reader.GetText(Position, @Buf, 8192);
      Inc(Position, NrRead);
      if (NrRead < 0) or (NrRead > 8192) then
        raise Exception.Create('Unexpected result from editor');
      Buf[NrRead] := #0;
      StrStrm.WriteString(PChar(@Buf));
    until NrRead < 8192;
    Result := StrStrm.DataString;
  finally
    StrStrm.Free;
  end;
  Reader := nil;
end; // func EditorBufAsString

//******************************************************************************
// Method Name:      GetCurrentBufferPos
// Input parameters: EditView: IOTAEditView interface to source editor
// Return value:     integer 0-based (TBC) offset of cursor in editor
// Exceptions:       ..  
// Side effects:     ..
// Conditions:       ..
// Description:      Returns offset index of current cursor position in editor
// Notes:            ..
//******************************************************************************
function GetCurrentBufferPos(EditView:IOTAEditView):integer;
var
  EditorPos: TOTAEditPos;
  CharPos: TOTACharPos;
begin
  Result:=0;
  if EditView <> nil then
  begin
    EditorPos := EditView.CursorPos;
    EditView.ConvertPos(True, EditorPos, CharPos);
    Result := EditView.CharPosToPos(CharPos);
  end;
end; // func GetCurrentBufferPos

//------------------------------------------------------------------------------
// Local expert declaration, hides class from outside access:
//------------------------------------------------------------------------------
type TFindCompRefWizard = class(TBaseWizard)
private
  procedure FindSelectedComponentInSource(Module:IOTAModule; FrmEditor:IOTAFormEditor);
  procedure FindSelectedComponentOnForm(Module:IOTAModule; SrcEditor:IOTASourceEditor);
public
  procedure Execute; override;
  function GetName: string; override;
end;

//******************************************************************************
// Method Name:      FindCompRefWizard
// Input parameters: None
// Return value:     IOTAWizard
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Wizard entry point, TFindCompRefWizard factory function
// Notes:            ..
//******************************************************************************
function FindCompRefWizard:IOTAWizard;
begin
  Result:=TFindCompRefWizard.Create;
end; // func PasLibImptWizard

//------------------------------------------------------------------------------
// TFindCompRefWizard implementation
//------------------------------------------------------------------------------

//******************************************************************************
// Method Name:      TFindCompRefWizard.GetName
// Input parameters: None
// Return value:     string Name
// Exceptions:       ..
// Side effects:     ..
// Conditions:       ..
// Description:      Mandatory implementation for GetName override
// Notes:            ..
//******************************************************************************
function TFindCompRefWizard.GetName:string;
begin
  Result:='FindCompRef Wizard';
end; // func GetName

//******************************************************************************
// Method Name:      TFindCompRefWizard.Execute
// Input parameters: None
// Exceptions:       ..
// Side effects:     ..
// Conditions:       From TCompRefMenuWizard.FindCompRefItemClick
// Description:      THIS IS WHERE IT ALL HAPPENS!
//                   Seeks the topmost editor in the IDE;
//                   - If it's a Form editor, invokes "Find comp ref in source".
//                   - If it's a Source editor, invokes "Find component in form"
// Notes:            ..
//******************************************************************************
procedure TFindCompRefWizard.Execute;
var Module: IOTAModule;
  CurEditor: IOTAEditor;
  FrmEditor: IOTAFormEditor;
  SrcEditor: IOTASourceEditor;
begin
  Module:=(BorlandIDEServices as IOTAModuleServices).CurrentModule;
  if not Assigned(Module) then Exit;
  CurEditor:=Module.GetCurrentEditor; // could be form designer or source editor
  if Supports(CurEditor,IOTAFormEditor,FrmEditor) then // it's a Form editor
    FindSelectedComponentInSource(Module,FrmEditor)
  else if Supports(CurEditor,IOTASourceEditor,SrcEditor) then // it's a Source editor
    FindSelectedComponentOnForm(Module,SrcEditor);
  // else: not interesting - forget it.
end; // proc Execute

//******************************************************************************
// Method Name:      TFindCompRefWizard.FindSelectedComponentOnForm
// Input parameters: Module: interface to current IDE module (project)
//                   SrcEditor: interface to current topmost source editor
// Return value:     None
// Exceptions:       ..
// Side effects:     ..
// Conditions:       Source editor showing when [Expert shortcut] is pressed.
// Description:      Determines if current source editor has associated form;
//                   if so, determines current identifier (at cursor) in editor,
//                   and whether this is the name of a component on the form.
//                   If the component is found, selects this component in the
//                   form editor and shows the form editor with selection.
// Notes:            ..
//******************************************************************************
procedure TFindCompRefWizard.FindSelectedComponentOnForm(Module:IOTAModule;
                             SrcEditor:IOTASourceEditor);
var SrcText,CurWord:string;
    CurrentPos,StartPos,IdentLength:integer;
    FrmEditor:IOTAFormEditor;
    Comp:IOTAComponent;
begin
  if UnitHasForm(Module,SrcEditor.FileName,FrmEditor) then
  begin
    SrcText:=EditorBufAsString(SrcEditor);
    CurrentPos:=GetCurrentBufferPos(SrcEditor.EditViews[0]);
    if CurrentIdent(SrcText,CurrentPos,StartPos,IdentLength) then
    begin
      CurWord := Copy(SrcText, StartPos, IdentLength);
      Comp:=FrmEditor.FindComponent(CurWord);
      if Comp<>nil then
      begin
        Comp.Select(False);
        FrmEditor.Show;
      end // if Comp<>nil
      else ShowMessage('No component ['+CurWord+'] Found')
    end
    else ShowMessage('No valid identifier')
  end
  else ShowMessage('No Associated Form for '+SrcEditor.FileName);
end; // proc FindSelectedComponentOnForm

//******************************************************************************
// Method Name:      TFindCompRefWizard.FindSelectedComponentInSource
// Input parameters: Module: interface to current IDE module (project)
//                   FrmEditor: interface to current topmost form editor
// Return value:     None
// Exceptions:       ..
// Side effects:     ..
// Conditions:       Form editor showing when [Expert shortcut] is pressed.
// Description:      Determines the name of the currently selected component
//                   in the form editor;
//                   Seeks the associated source unit in the source code editor,
//                   and searches/selects the *2nd* reference to this component
//                   in the source editor.
// Notes:            ..
//******************************************************************************
procedure TFindCompRefWizard.FindSelectedComponentInSource(Module:IOTAModule;
                             FrmEditor:IOTAFormEditor);
var ErrCode: integer;
  Comp: IOTAComponent;
  CompName :string;
  SrcEditor: IOTASourceEditor; // supports GetEditView(Index: Integer): IOTAEditView;
  CurIEDPos: IOTAEditPosition; // supports a Search method
begin
  //----------------------------------------------------------------------------
  // 1. Determine currently selected component, and its Name property
  //----------------------------------------------------------------------------
  Comp:=FrmEditor.GetSelComponent(0);
  if Comp<>nil then // could be that nothing is selected
  if Comp.GetPropValueByName('Name',CompName) then // Found name of selected component
  begin
    //--------------------------------------
    // 2. Find Source Editor:
    //--------------------------------------
    if FormHasUnit(Module,FrmEditor.FileName,SrcEditor) then
    begin
      SrcEditor.Show; // switches to Source editor
      //------------------------------------
      // 3. Initiate Search in Source Editor
      //------------------------------------
      CurIEDPos:=SrcEditor.EditViews[0].Position;
      //------------------------------------------------------------------------
      // DUMMY SEARCH: Based on b.p.d.opentoolsapi NG report from Neil Moss:
      // [Issue: IOTAEditPosition.Search does not respect WordBoundary property]
      // <quote> ... perform a 'dummy' search first (empty search expression)
      // that appears to initialise SearchOptions, then set WordBoundary
      // property, restore the cursor position to pre-search position and call
      // SearchAgain. Then the WordBoundary setting is acknowledged. </quote>
      //------------------------------------------------------------------------
      CurIEDPos.SearchOptions.SearchText:='';
      CurIEDPos.Search('',False,False,True,sdForward,ErrCode); // DUMMY
      CurIEDPos.Move(1,1); // move back to top
      // End of dummy search operation
      //-------------------------------

      //---------------------------------------------------
      // Do the REAL Search with proper search parameters:
      //---------------------------------------------------
      with CurIEDPos.SearchOptions do
      begin
        CaseSensitive:=False;
        Direction:=sdForward;
        WholeFile:=True;
        WordBoundary:=True;
        FromCursor:=False;
        RegularExpression:=False;
        SearchText:=CompName;
      end;
      CurIEDPos.SearchAgain; // skip to next ref in code
      CurIEDPos.SearchAgain; // skip to next ref in code
      SrcEditor.EditViews[0].Paint; // repaint editor view -> otherwise garbled
      // Known problem: F3 (for Search again) does not work to repeat
      // the component search after this... use Find Next/prev of GExperts instead.
      //-----------------------------------------------------------------
    end
    else ShowMessage('No Associated Unit for '+FrmEditor.FileName);
  end
  else ShowMessage('Cannot Find Name of Selected Component')
  else ; // do nothing - expert not applicable
end; // func FindSelectedComponentInSource


end.

//******************************************************************************
//*                          End of File FindCompWizard.pas
//******************************************************************************

